@@ -19,7 +19,6 @@
  * See `CHANGES' file for revision history.
  */
 
-#include "tacplus.h"
 #include "libtac.h"
 #include "xalloc.h"
 #include "md5.h"
@@ -35,12 +34,12 @@
  *             LIBTAC_STATUS_ASSEMBLY_ERR
  */
 int tac_authen_send(int fd, const char *user, char *pass, char *tty,
-    char *rem_addr) {
+    char *r_addr) {
 
     HDR *th;    /* TACACS+ packet header */
     struct authen_start tb;     /* message body */
     int user_len, port_len, chal_len, mdp_len, token_len, bodylength, w;
-    int rem_addr_len;
+    int r_addr_len;
     int pkt_len = 0;
     int ret = 0;
     char *chal = "1234123412341234";
@@ -52,7 +51,7 @@ int tac_authen_send(int fd, const char *user, char *pass, char *tty,
     th=_tac_req_header(TAC_PLUS_AUTHEN, 0);
 
     /* set some header options */
-    if ((tac_login != NULL) && (strcmp(tac_login,"login") == 0)) {
+    if (tac_login != NULL && !strcmp(tac_login,"login")) {
         th->version = TAC_PLUS_VER_0;
     } else {
         th->version = TAC_PLUS_VER_1;
@@ -60,10 +59,10 @@ int tac_authen_send(int fd, const char *user, char *pass, char *tty,
     th->encryption = tac_encryption ? TAC_PLUS_ENCRYPTED_FLAG : TAC_PLUS_UNENCRYPTED_FLAG;
 
     TACDEBUG((LOG_DEBUG, "%s: user '%s', tty '%s', rem_addr '%s', encrypt: %s", \
-        __FUNCTION__, user, tty, rem_addr, \
+        __FUNCTION__, user, tty, r_addr, \
         (tac_encryption) ? "yes" : "no"))        
         
-    if ((tac_login != NULL) && (strcmp(tac_login,"chap") == 0)) {
+    if (tac_login != NULL && !strcmp(tac_login,"chap")) {
         chal_len = strlen(chal);
         mdp_len = sizeof(u_char) + strlen(pass) + chal_len;
         mdp = (u_char *) xcalloc(1, mdp_len);
@@ -85,19 +84,19 @@ int tac_authen_send(int fd, const char *user, char *pass, char *tty,
     /* get size of submitted data */
     user_len = strlen(user);
     port_len = strlen(tty);
-    rem_addr_len = strlen(rem_addr);
+    r_addr_len = strlen(r_addr);
     token_len = strlen(token);
 
     /* fill the body of message */
     tb.action = TAC_PLUS_AUTHEN_LOGIN;
     tb.priv_lvl = tac_priv_lvl;
-    if (tac_login == NULL) {
+    if (tac_login == NULL || !*tac_login) {
         /* default to PAP */
         tb.authen_type = TAC_PLUS_AUTHEN_TYPE_PAP;
     } else {
-        if (strcmp(tac_login,"chap") == 0) {
+        if (!strcmp(tac_login,"chap")) {
             tb.authen_type = TAC_PLUS_AUTHEN_TYPE_CHAP;
-        } else if (strcmp(tac_login,"login") == 0) {
+        } else if (!strcmp(tac_login,"login")) {
             tb.authen_type = TAC_PLUS_AUTHEN_TYPE_ASCII;
         } else {
             tb.authen_type = TAC_PLUS_AUTHEN_TYPE_PAP;
@@ -106,12 +105,12 @@ int tac_authen_send(int fd, const char *user, char *pass, char *tty,
     tb.service = tac_authen_service;
     tb.user_len = user_len;
     tb.port_len = port_len;
-    tb.rem_addr_len = rem_addr_len;    /* may be e.g Caller-ID in future */
+    tb.r_addr_len = r_addr_len;    /* may be e.g Caller-ID in future */
     tb.data_len = token_len;
 
     /* fill body length in header */
     bodylength = sizeof(tb) + user_len
-        + port_len + rem_addr_len + token_len;
+        + port_len + r_addr_len + token_len;
 
     th->datalength = htonl(bodylength);
 
@@ -136,8 +135,8 @@ int tac_authen_send(int fd, const char *user, char *pass, char *tty,
     pkt_len += user_len;
     bcopy(tty, pkt+pkt_len, port_len);   /* tty */
     pkt_len += port_len;
-    bcopy(rem_addr, pkt+pkt_len, rem_addr_len);   /* rem addr */
-    pkt_len += rem_addr_len;
+    bcopy(r_addr, pkt+pkt_len, r_addr_len);   /* rem addr */
+    pkt_len += r_addr_len;
 
     bcopy(token, pkt+pkt_len, token_len);  /* password */
     pkt_len += token_len;
