@@ -19,6 +19,9 @@
  * See `CHANGES' file for revision history.
  */
 
+#include "pam_tacplus.h"
+#include "support.h"
+
 #include <stdlib.h>     /* malloc */
 #include <stdio.h>
 #include <syslog.h>
@@ -32,71 +35,30 @@
 #include <ctype.h>
 #include <time.h>
 #include <unistd.h>
-
-#ifndef __linux__
-    #include <strings.h>
-#endif
-
-#include "tacplus.h"
-#include "libtac.h"
-#include "pam_tacplus.h"
-#include "support.h"
-
-#define PAM_SM_AUTH
-#define PAM_SM_ACCOUNT
-#define PAM_SM_SESSION
-/* #define PAM_SM_PASSWORD */
-
-#ifndef __linux__
-    #include <security/pam_appl.h>
-#endif
-#include <security/pam_modules.h>
+#include <strings.h>
 
 #ifdef HAVE_CONFIG_H
-    #include "config.h"
+  #include "config.h"
 #endif
 
-/* support.c */
-extern struct addrinfo *tac_srv[TAC_PLUS_MAXSERVERS];
-extern char *tac_srv_key[TAC_PLUS_MAXSERVERS];
-extern int tac_srv_no;
-extern char *tac_service;
-extern char *tac_protocol;
-extern int _pam_parse (int argc, const char **argv);
-extern unsigned long _getserveraddr (char *serv);
-extern int tacacs_get_password (pam_handle_t * pamh, int flags
-    ,int ctrl, char **password);
-extern int converse (pam_handle_t * pamh, int nargs
-    ,struct pam_message **message
-    ,struct pam_response **response);
-extern void _pam_log (int err, const char *format,...);
-extern void *_xcalloc (size_t size);
-
-/* magic.c */
-extern u_int32_t magic();
-
 /* address of server discovered by pam_sm_authenticate */
-static struct addrinfo *active_server;
-char *active_key;
+static tacplus_server_t active_server;
+
 /* accounting task identifier */
 static short int task_id = 0;
 
 
 /* Helper functions */
 int _pam_send_account(int tac_fd, int type, const char *user, char *tty,
-    char *rem_addr, char *cmd) {
+    char *r_addr, char *cmd) {
 
-    char buf[40];
+    char buf[64];
     struct tac_attrib *attr;
     int retval;
-        
-    attr=(struct tac_attrib *)_xcalloc(sizeof(struct tac_attrib));
-        
-#ifdef _AIX
-    sprintf(buf, "%d", time(0));
-#else
-    sprintf(buf, "%lu", (long unsigned int)time(0));
-#endif
+
+    attr=(struct tac_attrib *)xcalloc(1, sizeof(struct tac_attrib));
+
+    sprintf(buf, "%lu", (unsigned long)time(NULL));
 
     if (type == TAC_PLUS_ACCT_FLAG_START) {
         tac_add_attrib(&attr, "start_time", buf);
@@ -111,7 +73,7 @@ int _pam_send_account(int tac_fd, int type, const char *user, char *tty,
         tac_add_attrib(&attr, "cmd", cmd);
     }
 
-    retval = tac_acct_send(tac_fd, type, user, tty, rem_addr, attr);
+    retval = tac_acct_send(tac_fd, type, user, tty, r_addr, attr);
 
     /* this is no longer needed */
     tac_free_attrib(&attr);
@@ -131,12 +93,17 @@ int _pam_send_account(int tac_fd, int type, const char *user, char *tty,
             __FUNCTION__, 
             tac_acct_flag2str(type),
             task_id);
-        if(re.msg != NULL) free(re.msg);
+
+        if(re.msg != NULL)
+            free(re.msg);
+
         close(tac_fd);
         return -1;
     }
 
-    if(re.msg != NULL) free(re.msg);
+    if(re.msg != NULL)
+        free(re.msg);
+
     close(tac_fd);
     return 0;
 }
@@ -148,7 +115,7 @@ int _pam_account(pam_handle_t *pamh, int argc, const char **argv,
     static int ctrl;
     char *user = NULL;
     char *tty = NULL;
-    char *rem_addr = NULL;
+    char *r_addr = NULL;
     char *typemsg;
     int status = PAM_SESSION_ERR;
   
@@ -156,7 +123,7 @@ int _pam_account(pam_handle_t *pamh, int argc, const char **argv,
     ctrl = _pam_parse (argc, argv);
 
     if (ctrl & PAM_TAC_DEBUG)
-        syslog (LOG_DEBUG, "%s: [%s] called (pam_tacplus v%hu.%hu.%hu)"
+        syslog (LOG_DEBUG, "%s: [%s] called (pam_tacplus v%u.%u.%u)"
             , __FUNCTION__, typemsg, PAM_TAC_VMAJ, PAM_TAC_VMIN, PAM_TAC_VPAT);
     if (ctrl & PAM_TAC_DEBUG)
         syslog(LOG_DEBUG, "%s: tac_srv_no=%d", __FUNCTION__, tac_srv_no);
@@ -173,9 +140,9 @@ int _pam_account(pam_handle_t *pamh, int argc, const char **argv,
     if (ctrl & PAM_TAC_DEBUG)
         syslog(LOG_DEBUG, "%s: tty [%s] obtained", __FUNCTION__, tty);
 
-    rem_addr = _pam_get_rhost(pamh);
+    r_addr = _pam_get_rhost(pamh);
     if (ctrl & PAM_TAC_DEBUG)
-        syslog(LOG_DEBUG, "%s: rhost [%s] obtained", __FUNCTION__, rem_addr);
+        syslog(LOG_DEBUG, "%s: rhost [%s] obtained", __FUNCTION__, r_addr);
 
     /* checks for specific data required by TACACS+, which should
        be supplied in command line  */
@@ -207,7 +174,7 @@ int _pam_account(pam_handle_t *pamh, int argc, const char **argv,
         while ((status == PAM_SESSION_ERR) && (srv_i < tac_srv_no)) {
             int tac_fd;
                                   
-            tac_fd = tac_connect_single(tac_srv[srv_i], tac_srv_key[srv_i]);
+            tac_fd = tac_connect_single(tac_srv[srv_i].addr, tac_srv[srv_i].key);
             if(tac_fd < 0) {
                 _pam_log(LOG_WARNING, "%s: error sending %s (fd)",
                     __FUNCTION__, typemsg);
@@ -218,7 +185,7 @@ int _pam_account(pam_handle_t *pamh, int argc, const char **argv,
             if (ctrl & PAM_TAC_DEBUG)
                 syslog(LOG_DEBUG, "%s: connected with fd=%d (srv %d)", __FUNCTION__, tac_fd, srv_i);
 
-            retval = _pam_send_account(tac_fd, type, user, tty, rem_addr, cmd);
+            retval = _pam_send_account(tac_fd, type, user, tty, r_addr, cmd);
             /* return code from function in this mode is
                status of the last server we tried to send
                packet to */
@@ -242,7 +209,7 @@ int _pam_account(pam_handle_t *pamh, int argc, const char **argv,
         for(srv_i = 0; srv_i < tac_srv_no; srv_i++) {
             int tac_fd;
                                   
-            tac_fd = tac_connect_single(tac_srv[srv_i], tac_srv_key[srv_i]);
+            tac_fd = tac_connect_single(tac_srv[srv_i].addr, tac_srv[srv_i].key);
             if(tac_fd < 0) {
                 _pam_log(LOG_WARNING, "%s: error sending %s (fd)",
                     __FUNCTION__, typemsg);
@@ -252,7 +219,7 @@ int _pam_account(pam_handle_t *pamh, int argc, const char **argv,
             if (ctrl & PAM_TAC_DEBUG)
                 syslog(LOG_DEBUG, "%s: connected with fd=%d (srv %d)", __FUNCTION__, tac_fd, srv_i);
 
-            retval = _pam_send_account(tac_fd, type, user, tty, rem_addr, cmd);
+            retval = _pam_send_account(tac_fd, type, user, tty, r_addr, cmd);
             /* return code from function in this mode is
                status of the last server we tried to send
                packet to */
@@ -292,17 +259,17 @@ int pam_sm_authenticate (pam_handle_t * pamh, int flags,
     char *user;
     char *pass;
     char *tty;
-    char *rem_addr;
+    char *r_addr;
     int srv_i;
     int tac_fd;
     int status = PAM_AUTH_ERR;
 
-    user = pass = tty = rem_addr = NULL;
+    user = pass = tty = r_addr = NULL;
 
     ctrl = _pam_parse (argc, argv);
 
     if (ctrl & PAM_TAC_DEBUG)
-        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%hu.%hu.%hu)"
+        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%u.%u.%u)"
             , __FUNCTION__, PAM_TAC_VMAJ, PAM_TAC_VMIN, PAM_TAC_VPAT);
 
     if ((user = _pam_get_user(pamh)) == NULL)
@@ -334,16 +301,16 @@ int pam_sm_authenticate (pam_handle_t * pamh, int flags,
     if (ctrl & PAM_TAC_DEBUG)
         syslog (LOG_DEBUG, "%s: tty [%s] obtained", __FUNCTION__, tty);
 
-    rem_addr = _pam_get_rhost(pamh);
+    r_addr = _pam_get_rhost(pamh);
     if (ctrl & PAM_TAC_DEBUG)
-        syslog (LOG_DEBUG, "%s: rhost [%s] obtained", __FUNCTION__, rem_addr);
+        syslog (LOG_DEBUG, "%s: rhost [%s] obtained", __FUNCTION__, r_addr);
 
     for (srv_i = 0; srv_i < tac_srv_no; srv_i++) {
         int msg = TAC_PLUS_AUTHEN_STATUS_FAIL;
         if (ctrl & PAM_TAC_DEBUG)
             syslog (LOG_DEBUG, "%s: trying srv %d", __FUNCTION__, srv_i );
 
-        tac_fd = tac_connect_single(tac_srv[srv_i], tac_srv_key[srv_i]);
+        tac_fd = tac_connect_single(tac_srv[srv_i].addr, tac_srv[srv_i].key);
         if (tac_fd < 0) {
             _pam_log (LOG_ERR, "connection failed srv %d: %m", srv_i);
             if (srv_i == tac_srv_no-1) {
@@ -353,7 +320,7 @@ int pam_sm_authenticate (pam_handle_t * pamh, int flags,
             continue;
         }
 
-        if (tac_authen_send(tac_fd, user, pass, tty, rem_addr) < 0) {
+        if (tac_authen_send(tac_fd, user, pass, tty, r_addr) < 0) {
             _pam_log (LOG_ERR, "error sending auth req to TACACS+ server");
             status = PAM_AUTHINFO_UNAVAIL;
         } else {
@@ -373,9 +340,13 @@ int pam_sm_authenticate (pam_handle_t * pamh, int flags,
                         /* OK, we got authenticated; save the server that
                            accepted us for pam_sm_acct_mgmt and exit the loop */
                         status = PAM_SUCCESS;
-                        active_server = tac_srv[srv_i];
-                        active_key = tac_srv_key[srv_i];
+                        active_server.addr = tac_srv[srv_i].addr;
+                        active_server.key = tac_srv[srv_i].key;
                         close(tac_fd);
+
+                        if (ctrl & PAM_TAC_DEBUG)
+                            syslog (LOG_DEBUG, "%s: active srv %d", __FUNCTION__, srv_i );
+
                         break;
                     }
                 }
@@ -386,9 +357,13 @@ int pam_sm_authenticate (pam_handle_t * pamh, int flags,
                 /* OK, we got authenticated; save the server that
                    accepted us for pam_sm_acct_mgmt and exit the loop */
                 status = PAM_SUCCESS;
-                active_server = tac_srv[srv_i];
-                active_key = tac_srv_key[srv_i];
+                active_server.addr = tac_srv[srv_i].addr;
+                active_server.key = tac_srv[srv_i].key;
                 close(tac_fd);
+
+                if (ctrl & PAM_TAC_DEBUG)
+                    syslog (LOG_DEBUG, "%s: active srv %d", __FUNCTION__, srv_i );
+
                 break;
             }
         }
@@ -414,7 +389,7 @@ int pam_sm_setcred (pam_handle_t * pamh, int flags,
     int ctrl = _pam_parse (argc, argv);
 
     if (ctrl & PAM_TAC_DEBUG)
-        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%hu.%hu.%hu)"
+        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%u.%u.%u)"
             , __FUNCTION__, PAM_TAC_VMAJ, PAM_TAC_VMIN, PAM_TAC_VPAT);
 
     return PAM_SUCCESS;
@@ -432,12 +407,12 @@ int pam_sm_acct_mgmt (pam_handle_t * pamh, int flags,
     int retval, ctrl, status=PAM_AUTH_ERR;
     char *user;
     char *tty;
-    char *rem_addr;
+    char *r_addr;
     struct areply arep;
     struct tac_attrib *attr = NULL;
     int tac_fd;
 
-    user = tty = rem_addr = NULL;
+    user = tty = r_addr = NULL;
   
     /* this also obtains service name for authorization
        this should be normally performed by pam_get_item(PAM_SERVICE)
@@ -447,7 +422,7 @@ int pam_sm_acct_mgmt (pam_handle_t * pamh, int flags,
     ctrl = _pam_parse (argc, argv);
 
     if (ctrl & PAM_TAC_DEBUG)
-        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%hu.%hu.%hu)"
+        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%u.%u.%u)"
             , __FUNCTION__, PAM_TAC_VMAJ, PAM_TAC_VMIN, PAM_TAC_VPAT);
   
     if ((user = _pam_get_user(pamh)) == NULL)
@@ -455,36 +430,36 @@ int pam_sm_acct_mgmt (pam_handle_t * pamh, int flags,
 
     if (ctrl & PAM_TAC_DEBUG)
         syslog(LOG_DEBUG, "%s: username obtained [%s]", __FUNCTION__, user);
-  
+ 
     tty = _pam_get_terminal(pamh);
     if(!strncmp(tty, "/dev/", 5)) 
         tty += 5;
     if (ctrl & PAM_TAC_DEBUG)
         syslog(LOG_DEBUG, "%s: tty obtained [%s]", __FUNCTION__, tty);
 
-    rem_addr = _pam_get_rhost(pamh);
+    r_addr = _pam_get_rhost(pamh);
     if (ctrl & PAM_TAC_DEBUG)
-        syslog(LOG_DEBUG, "%s: rhost obtained [%s]", __FUNCTION__, rem_addr);
+        syslog(LOG_DEBUG, "%s: rhost obtained [%s]", __FUNCTION__, r_addr);
   
     /* checks if user has been successfully authenticated
        by TACACS+; we cannot solely authorize user if it hasn't
        been authenticated or has been authenticated by method other
        than TACACS+ */
-    if(!active_server) {
+    if(active_server.addr == NULL) {
         _pam_log (LOG_ERR, "user not authenticated by TACACS+");
         return PAM_AUTH_ERR;
     }
     if (ctrl & PAM_TAC_DEBUG)
         syslog (LOG_DEBUG, "%s: active server is [%s]", __FUNCTION__,
-            tac_ntop(active_server->ai_addr, active_server->ai_addrlen));
+            tac_ntop(active_server.addr->ai_addr));
 
     /* checks for specific data required by TACACS+, which should
        be supplied in command line  */
-    if(tac_service == NULL || *tac_service == '\0') {
+    if(tac_service == NULL || !*tac_service) {
         _pam_log (LOG_ERR, "TACACS+ service type not configured");
         return PAM_AUTH_ERR;
     }
-    if(tac_protocol == NULL || *tac_protocol == '\0') {
+    if(tac_protocol == NULL || !*tac_protocol) {
         _pam_log (LOG_ERR, "TACACS+ protocol type not configured");
         return PAM_AUTH_ERR;
     }
@@ -492,21 +467,25 @@ int pam_sm_acct_mgmt (pam_handle_t * pamh, int flags,
     tac_add_attrib(&attr, "service", tac_service);
     tac_add_attrib(&attr, "protocol", tac_protocol);
 
-    tac_fd = tac_connect_single(active_server, active_key);
+    tac_fd = tac_connect_single(active_server.addr, active_server.key);
     if(tac_fd < 0) {
         _pam_log (LOG_ERR, "TACACS+ server unavailable");
-        if(arep.msg != NULL) free (arep.msg);
+        if(arep.msg != NULL)
+            free (arep.msg);
+
         close(tac_fd);
         return PAM_AUTH_ERR;
     }
 
-    retval = tac_author_send(tac_fd, user, tty, rem_addr, attr);
+    retval = tac_author_send(tac_fd, user, tty, r_addr, attr);
 
     tac_free_attrib(&attr);
   
     if(retval < 0) {
         _pam_log (LOG_ERR, "error getting authorization");
-        if(arep.msg != NULL) free (arep.msg);
+        if(arep.msg != NULL)
+            free (arep.msg);
+
         close(tac_fd);
         return PAM_AUTH_ERR;
     }
@@ -520,7 +499,9 @@ int pam_sm_acct_mgmt (pam_handle_t * pamh, int flags,
         arep.status != AUTHOR_STATUS_PASS_REPL) {
 
         _pam_log (LOG_ERR, "TACACS+ authorisation failed for [%s]", user);
-        if(arep.msg != NULL) free (arep.msg);
+        if(arep.msg != NULL)
+            free (arep.msg);
+
         close(tac_fd);
         return PAM_PERM_DENIED;
     }
@@ -566,8 +547,12 @@ int pam_sm_acct_mgmt (pam_handle_t * pamh, int flags,
     }
 
     /* free returned attributes */
-    if(arep.attr != NULL) tac_free_attrib(&arep.attr);
-    if(arep.msg != NULL) free (arep.msg);
+    if(arep.attr != NULL)
+        tac_free_attrib(&arep.attr);
+
+    if(arep.msg != NULL)
+        free (arep.msg);
+
     close(tac_fd);
 
     return status;
@@ -610,7 +595,7 @@ int pam_sm_chauthtok (pam_handle_t * pamh, int flags,
     int ctrl = _pam_parse (argc, argv);
 
     if (ctrl & PAM_TAC_DEBUG)
-        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%hu.%hu.%hu)"
+        syslog (LOG_DEBUG, "%s: called (pam_tacplus v%u.%u.%u)"
             , __FUNCTION__, PAM_TAC_VMAJ, PAM_TAC_VMIN, PAM_TAC_VPAT);
 
     return PAM_SUCCESS;
